// RobotControl.java
// Set portName as appropriate

import ch.aplu.util.Console;
import ch.aplu.util.*;
import java.io.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;
import javax.comm.*;
import java.util.*;

public class RobotControl extends GPanel
        implements MouseListener, MouseMotionListener,
                   SerialPortEventListener
{
  private final String portName = "COM1";
  private double radius = 8;
  private double smallStep = 0.5;
  private double bigStep = 5;
  private Point2D.Double soll = new Point2D.Double(0, 0);
  private Point2D.Double ist = new Point2D.Double(0, 0);
  private PrintStream ps;
  private BufferedReader br;
  private boolean isHot = false;
  private boolean isOutside = false;

  public RobotControl() throws IOException
  {
    super("Robot Control - Drag red point to new location",
          -200, 200, -200, 200);
    resizable(false);
    draw(ist, soll);
    addMouseListener(this);
    addMouseMotionListener(this);
    SerialPort port = PortCreator.openPort(portName);
    if (port == null)
    {
      System.out.println("Cannot open Com port");
      System.exit(1);
    }
    ps = new PrintStream(port.getOutputStream());
    br = new BufferedReader(
      new InputStreamReader(port.getInputStream()));
    try
    {
      port.addEventListener(this);
    }
    catch (TooManyListenersException ex) {}
    port.notifyOnDataAvailable(true);

    while (true)
    {
      Console.delay(200);
      double step;
      ps.println(getDirection(ist, soll) + ";" +
                 adaptStep(ist, soll));
    }
  }

  public void mousePressed(MouseEvent evt)
  {
    Point2D.Double pt = toWindow(evt.getPoint());
    if (isHot)
    {
      soll = pt;
      draw(ist, soll);
      getPane().setCursor(new Cursor(Cursor.CROSSHAIR_CURSOR));
    }
  }

  public void mouseReleased(MouseEvent evt)
  {
    Point2D.Double pt = toWindow(evt.getPoint());
    if (isHot)
      getPane().setCursor(new Cursor(Cursor.HAND_CURSOR));
  }

  public void mouseMoved(MouseEvent evt)
  {
    Point2D.Double pt = toWindow(evt.getPoint());
    if (checkHot(pt, soll))
    {
      getPane().setCursor(new Cursor(Cursor.HAND_CURSOR));
      isHot = true;
    }
    else
    {
      getPane().setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
      isHot = false;
    }
  }

  public void mouseDragged(MouseEvent evt)
  {
    if (isOutside)
      return;
    Point2D.Double pt = toWindow(evt.getPoint());
    if (isHot)
    {
      soll = pt;
      draw(ist, soll);
    }
    else
      if (checkHot(pt, soll))
      {
        getPane().setCursor(new Cursor(Cursor.CROSSHAIR_CURSOR));
        isHot = true;
      }
  }

  public void mouseEntered(MouseEvent evt)
  {
    isOutside = false;
  }

  public void mouseExited(MouseEvent evt)
  {
    isOutside = true;
  }

  public void mouseClicked(MouseEvent evt) {}

  private boolean checkHot(Point2D.Double p1,
                           Point2D.Double p2)
  {
    return distance(p1, p2) < radius;
  }

  private double distance(Point2D.Double p1,
                          Point2D.Double p2)
  {
    return Math.sqrt((p1.x - p2.x) * (p1.x - p2.x) +
                     (p1.y - p2.y) * (p1.y - p2.y));
  }

  synchronized public void draw(Point2D.Double ist,
                                Point2D.Double soll)
  {
    clear();
    color(Color.green);
    move(ist);
    fillCircle(radius);
    color(Color.red);
    move(soll);
    fillCircle(radius);
  }

  public double getDirection(Point2D.Double ist,
                             Point2D.Double soll)
  {
    double dir = 180.0 / Math.PI *
                 Math.atan2(soll.y - ist.y, soll.x - ist.x);
    return 90 - dir;
  }

  public double adaptStep(Point2D.Double ist,
                          Point2D.Double soll)
  {
    double d = distance(ist, soll);
    if (d > 3 * radius)
      return bigStep;
    else
      return smallStep;
  }

  public void serialEvent(SerialPortEvent evt)
  {
    if (evt.getEventType() == SerialPortEvent.DATA_AVAILABLE)
    {
      try
      {
        String data = br.readLine();
        int k = data.indexOf(';');
        ist.x = Double.parseDouble(data.substring(0, k));
        ist.y = Double.parseDouble(data.substring(k + 1));
        draw(ist, soll);
      }
      catch (Exception ex) {}
    }
  }

  public static void main(String[] args) throws IOException
  {
    new RobotControl();
  }
}
